/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator;

import java.util.Objects;

import org.apache.commons.collections.MapUtils;
import org.springframework.stereotype.Component;

import de.ozgcloud.operator.PutRoleRequestData.IndicesPrivilegesData;
import de.ozgcloud.operator.common.elasticsearch.ElasticsearchRemoteService;
import de.ozgcloud.operator.common.kubernetes.KubernetesRemoteService;
import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.client.dsl.Resource;
import io.fabric8.kubernetes.client.extension.ResourceAdapter;
import io.javaoperatorsdk.operator.api.reconciler.Context;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RequiredArgsConstructor
@Component
public class OzgCloudElasticsearchService {

	private final OzgCloudElasticsearchSecretHelper secretHelper;
	private final OzgCloudElasticsearchProperties properties;

	private final ElasticsearchRemoteService remoteService;

	private final KubernetesRemoteService kubernetesService;

	public Secret getOrCreateCredentialSecret(OzgCloudElasticsearchCustomResource resource, Context<OzgCloudElasticsearchCustomResource> context) {
		try {
			LOG.debug("{}: Get or create secret.", resource.getMetadata().getNamespace());
			var namespace = resource.getMetadata().getNamespace();
			var secretResource = getCredentialsSecretResource(namespace);

			if (Objects.isNull(secretResource.get())) {
				LOG.info("{}: Secret not exists, create one ...", resource.getMetadata().getNamespace());
				createCredentialSecret(secretResource, namespace);
				LOG.info("{}: Secret creation successful.", resource.getMetadata().getNamespace());
			}
			return secretResource.get();
		} catch (Exception e) {
			LOG.warn(resource.getMetadata().getNamespace() + ": Secret creation failed: ", e);
			throw e;
		}
	}

	private Resource<Secret> getCredentialsSecretResource(String namespace) {
		return kubernetesService.getSecretResource(namespace, properties.getSecretCredentialsName());
	}

	private void createCredentialSecret(Resource<Secret> resource, String namespace) {
		createAdapter(resource).create(secretHelper.buildCredentialSecret(namespace, properties.getSecretCredentialsName()));
	}

	public void createIndexIfMissing(String name) throws Exception {
		LOG.debug("{}: Check elasticsearch index...", name);
		if (!remoteService.existsIndex(name)) {
			remoteService.createIndex(name);
		}
	}

	public void createSecurityRoleIfMissing(String roleName) throws Exception {
		remoteService.createOrUpdateSecurityRole(buildPutRoleRequestData(roleName));
	}

	PutRoleRequestData buildPutRoleRequestData(String roleName) {
		return PutRoleRequestData.builder().name(roleName).indivesPrivilegesData(buildIndicesPrivilegesData(roleName)).build();
	}

	private IndicesPrivilegesData buildIndicesPrivilegesData(String roleName) {
		return IndicesPrivilegesData.builder().names(roleName).privileges(IndicesPrivilege.ALL.getValue()).build();
	}

	public void createSecurityUserIfMissing(String namespace, String password) throws Exception {
		remoteService.createOrUpdateSecurityUser(buildPutUserRequestData(namespace, password));
	}

	PutUserRequestData buildPutUserRequestData(String namespace, String password) {
		return PutUserRequestData.builder().username(namespace).roles(namespace).password(password).build();
	}

	public void deleteSecurityUserIfExists(String userName) throws Exception {
		remoteService.deleteSecurityUser(userName);
	}

	public void deleteSecurityRoleIfExists(String roleName) throws Exception {
		remoteService.deleteSecurityRole(roleName);
	}

	public void deleteIndexIfExists(String indexName) throws Exception {
		LOG.debug("{}: Check delete elasticsearch index ...", indexName);
		if (remoteService.existsIndex(indexName)) {
			remoteService.deleteIndex(indexName);
		}
	}

	public void createCertificateIfMissing(String namespace) {
		try {
			LOG.debug("{}: Create certificate secret if missing...", namespace);
			var secretResource = kubernetesService.getSecretResource(namespace, properties.getCertificateSecretName());

			if (Objects.isNull(secretResource.get())) {
				LOG.info("{}: Create certificate secret", namespace);
				createCredentialSecret(namespace, secretResource);
			}
		} catch (Exception e) {
			throw new RuntimeException("Certificate secret creation failed " + namespace, e);
		}
	}

	void createCredentialSecret(String namespace, Resource<Secret> secretResource) {
		var serverSecretResource = kubernetesService.getSecretResource(properties.getServer().getCertificateNamespace(),
				properties.getServer().getCertificateSecretName());

		createAdapter(secretResource).create(secretHelper.buildCertificateSecret(namespace, getSecretData(serverSecretResource.get())));
	}

	private String getSecretData(Secret secret) {
		return MapUtils.getString(secret.getData(), properties.getServer().getCertificateSecretDataKey());
	}

	ResourceAdapter<Secret> createAdapter(Resource<Secret> resource) {
		return new ResourceAdapter<>(resource);
	}
}