/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.common.elasticsearch;

import java.io.IOException;

import org.springframework.stereotype.Component;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch._types.ElasticsearchException;
import co.elastic.clients.elasticsearch.security.IndicesPrivileges;
import co.elastic.clients.elasticsearch.security.PutRoleRequest;
import co.elastic.clients.elasticsearch.security.PutUserRequest;
import de.ozgcloud.operator.PutRoleRequestData;
import de.ozgcloud.operator.PutUserRequestData;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RequiredArgsConstructor
@Component
public class ElasticsearchRemoteService {

	private final ElasticsearchClient client;

	public boolean existsIndex(String index) throws Exception {
		try {
			LOG.debug("{}: Test if elasticsearch index exits.", index);
			var exists = client.indices().exists(builder -> builder.index(index)).value();
			LOG.debug("{}: Elasticsearch index exists: {}", index, exists);
			return exists;
		} catch (ElasticsearchException | IOException e) {
			throw new RuntimeException("Error checking index " + index, e);
		}
	}

	public void createIndex(String indexName) throws Exception {
		try {
			LOG.info("{}: Create or update elasticsearch index", indexName);
			client.indices().create(builder -> builder.index(indexName));
			LOG.info("{}: Create or update elasticsearch index successful", indexName);
		} catch (Exception e) {
			throw new RuntimeException("Create elasticsearch index " + indexName + " failed.", e);
		}
	}

	public boolean existsSecurityRole(String roleName) throws Exception {
		return !client.security().getRole(builder -> builder.name(roleName)).result().isEmpty();
	}

	public void createOrUpdateSecurityRole(PutRoleRequestData requestData) throws Exception {
		try {
			LOG.info("{}: Create or update elasticsearch role ", requestData.getName());
			client.security().putRole(createPutRoleRequest(requestData));
			LOG.info("{}: Create or update elasticsearch role successful", requestData.getName());
		} catch (Exception e) {
			throw new RuntimeException("Create elasticsearch role " + requestData.getName() + " failed.", e);
		}
	}

	PutRoleRequest createPutRoleRequest(PutRoleRequestData requestData) {
		return PutRoleRequest.of(requestBuilder -> buildRequest(requestBuilder, requestData));
	}

	private PutRoleRequest.Builder buildRequest(PutRoleRequest.Builder requestBuilder, PutRoleRequestData requestData) {
		requestBuilder.name(requestData.getName());
		requestBuilder.indices(builder -> buildIndicesPrivilegesRequest(builder, requestData));

		return requestBuilder;
	}

	private IndicesPrivileges.Builder buildIndicesPrivilegesRequest(IndicesPrivileges.Builder builder, PutRoleRequestData requestData) {
		builder.names(requestData.getIndivesPrivilegesData().getNames());
		builder.privileges(requestData.getIndivesPrivilegesData().getPrivileges());

		return builder;
	}

	public boolean existsSecurityUser(String userName) throws Exception {
		return !client.security().getUser(builder -> builder.username(userName)).result().isEmpty();
	}

	public void createOrUpdateSecurityUser(PutUserRequestData requestData) throws Exception {
		try {
			LOG.info("{}: Create or update elasticsearch user", requestData.getUsername());
			client.security().putUser(createPutUserRequest(requestData));
			LOG.info("{}: Create or update elasticsearch user successful", requestData.getUsername());
		} catch (Exception e) {
			throw new RuntimeException("Create elasticsearch user " + requestData.getUsername() + " failed.", e);
		}
	}

	PutUserRequest createPutUserRequest(PutUserRequestData requestData) {
		return PutUserRequest.of(requestBuilder -> buildPutUserRequest(requestBuilder, requestData));
	}

	private PutUserRequest.Builder buildPutUserRequest(PutUserRequest.Builder builder, PutUserRequestData requestData) {
		builder.username(requestData.getUsername());
		builder.roles(requestData.getRoles());
		builder.password(requestData.getPassword());

		return builder;
	}

	public void deleteIndex(String indexName) throws Exception {
		try {
			LOG.info("{}: Delete elasticsearch index if exists", indexName);
			client.indices().delete(builder -> builder.index(indexName));
			LOG.info("{}: Delete elasticsearch index if exists successful", indexName);
		} catch (Exception e) {
			throw new RuntimeException("Delete elasticsearch index " + indexName + " failed.", e);
		}
	}

	public void deleteSecurityRole(String roleName) throws Exception {
		try {
			LOG.info("{}: Delete elasticsearch role if exists", roleName);
			client.security().deleteRole(builder -> builder.name(roleName));
			LOG.info("{}: Delete elasticsearch role if exists successful", roleName);
		} catch (Exception e) {
			throw new RuntimeException("Delete elasticsearch role " + roleName + " failed.", e);
		}
	}

	public void deleteSecurityUser(String userName) throws Exception {
		try {
			LOG.info("{}: Delete elasticsearch user if exists", userName);
			client.security().deleteUser(builder -> builder.username(userName));
			LOG.info("{}: Delete elasticsearch user if exists successful", userName);
		} catch (Exception e) {
			throw new RuntimeException("Delete elasticsearch user " + userName + " failed.", e);
		}
	}
}