/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.operator.OzgCloudElasticsearchProperties.OzgCloudElasticsearchServerProperties;
import de.ozgcloud.operator.common.kubernetes.NamespaceTestFactory;
import de.ozgcloud.operator.common.kubernetes.SecretTestFactory;
import io.fabric8.kubernetes.api.model.Secret;

class OzgCloudElasticsearchSecretHelperTest {

	@Spy
	@InjectMocks
	private OzgCloudElasticsearchSecretHelper builder;
	@Mock
	private OzgCloudElasticsearchProperties properties;

	@DisplayName("Build credential secret")
	@Nested
	class TestBuildCredentialsSecret {

		private final static String HOST = "dummyHost";
		private final static int PORT = 42;

		@Mock
		private OzgCloudElasticsearchServerProperties serverProperties;

		@BeforeEach
		void mockProperties() {
			when(properties.getServer()).thenReturn(serverProperties);
			when(serverProperties.getHost()).thenReturn(HOST);
			when(serverProperties.getPort()).thenReturn(PORT);
		}

		@Test
		void shouldContainType() {
			var secret = buildCredentialSecret();

			assertThat(secret.getType()).isEqualTo(OzgCloudElasticsearchSecretHelper.SECRET_TYPE);
		}

		@DisplayName("metadata")
		@Nested
		class TestMetadata {

			@Test
			void shouldContainName() {
				var secret = buildCredentialSecret();

				assertThat(secret.getMetadata().getName()).isEqualTo(SecretTestFactory.NAME);
			}

			@Test
			void shouldContainNamespace() {
				var secret = buildCredentialSecret();

				assertThat(secret.getMetadata().getNamespace()).isEqualTo(NamespaceTestFactory.NAMESPACE);
			}
		}

		@DisplayName("address")
		@Nested
		class TestAddress {

			@Test
			void shouldBeSet() {
				var secret = buildCredentialSecret();

				assertThat(secret.getStringData()).containsEntry(OzgCloudElasticsearchSecretHelper.CREDENTIAL_SECRET_ADDRESS_FIELD,
						String.format("%s:%s", HOST, PORT));
			}

			@Test
			void shouldGetHostFromProperties() {
				buildCredentialSecret();

				verify(serverProperties).getHost();
			}

			@Test
			void shouldGetPortFromPorperties() {
				buildCredentialSecret();

				verify(serverProperties).getPort();
			}
		}

		@Test
		void shouldContainIndex() {
			var secret = buildCredentialSecret();

			assertThat(secret.getStringData()).containsEntry(OzgCloudElasticsearchSecretHelper.CREDENTIAL_SECRET_INDEX_FIELD,
					NamespaceTestFactory.NAMESPACE);
		}

		@Test
		void shouldContainPassword() {
			var secret = buildCredentialSecret();

			assertThat(secret.getStringData()).containsKey(OzgCloudElasticsearchSecretHelper.CREDENTIAL_SECRET_PASSWORD_FIELD);
			assertThat(secret.getStringData().get(OzgCloudElasticsearchSecretHelper.CREDENTIAL_SECRET_PASSWORD_FIELD)).isNotNull();
		}

		@Test
		void shouldContainUsername() {
			var secret = buildCredentialSecret();

			assertThat(secret.getStringData()).containsEntry(OzgCloudElasticsearchSecretHelper.CREDENTIAL_SECRET_USERNAME_FIELD,
					NamespaceTestFactory.NAMESPACE);
		}

		private Secret buildCredentialSecret() {
			return builder.buildCredentialSecret(NamespaceTestFactory.NAMESPACE, SecretTestFactory.NAME);
		}
	}

	@DisplayName("Build certificate secret")
	@Nested
	class TestBuildCertificatSecret {

		private static final String DATA = "fgdrgsgreg";

		@Test
		void shouldHaveType() {
			var secret = builder.buildCertificateSecret(NamespaceTestFactory.NAMESPACE, DATA);

			assertThat(secret.getType()).isEqualTo(OzgCloudElasticsearchSecretHelper.SECRET_TYPE);
		}

		@DisplayName("metadata")
		@Nested
		class TestMetadata {

			private static final String CERTIFICATE_SECRET_NAME = "rfgsgrgsr";

			@BeforeEach
			void mock() {
				when(properties.getCertificateSecretName()).thenReturn(CERTIFICATE_SECRET_NAME);
			}

			@Test
			void shouldContainName() {
				var secret = builder.buildCertificateSecret(NamespaceTestFactory.NAMESPACE, DATA);

				assertThat(secret.getMetadata().getName()).isEqualTo(CERTIFICATE_SECRET_NAME);
			}

			@Test
			void shouldContainNamespace() {
				var secret = builder.buildCertificateSecret(NamespaceTestFactory.NAMESPACE, DATA);

				assertThat(secret.getMetadata().getNamespace()).isEqualTo(NamespaceTestFactory.NAMESPACE);
			}
		}

		@Test
		void shouldHaveCaCrtData() {
			var secret = builder.buildCertificateSecret(NamespaceTestFactory.NAMESPACE, DATA);

			assertThat(secret.getData()).containsExactly(Map.entry(OzgCloudElasticsearchSecretHelper.CERTIFICATE_SECRET_DATA_KEY, DATA));
		}
	}
}
