/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.common.elasticsearch;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch.indices.ExistsRequest;
import co.elastic.clients.elasticsearch.security.IndicesPrivileges;
import co.elastic.clients.elasticsearch.security.PutRoleRequest;
import co.elastic.clients.elasticsearch.security.PutUserRequest;
import de.ozgcloud.operator.IndicesPrivilegesDataTestFactory;
import de.ozgcloud.operator.PutRoleRequestDataTestFactory;
import de.ozgcloud.operator.PutUserRequestDataTestFactory;
import lombok.SneakyThrows;

class ElasticsearchRemoteServiceITCase {

	private static final String INDEX_NAME = "test_index";

	private final ElasticsearchClient client = ElasticsearchTestClient.create();
	private final ElasticsearchRemoteService service = new ElasticsearchRemoteService(client);

	@BeforeAll
	public static void startContainer() {
		ElasticsearchTestClient.ELASTICSEARCH_CONTAINER.start();
	}

	@AfterAll
	public static void stopContainer() {
		ElasticsearchTestClient.ELASTICSEARCH_CONTAINER.stop();
	}

	@DisplayName("Exists index")
	@Nested
	class TestExistsIndex {

		@DisplayName("on existing")
		@Nested
		class TestOnExisting {

			@SneakyThrows
			@BeforeEach
			public void initIndex() {
				createIndex();
			}

			@AfterEach
			void cleanup() {
				deleteIndex();
			}

			@SneakyThrows
			@Test
			void shouldReturnTrue() {
				var exists = service.existsIndex(INDEX_NAME);

				assertThat(exists).isTrue();
			}
		}

		@SneakyThrows
		@Test
		void shouldReturnFalseIfMissing() {
			var exists = service.existsIndex(INDEX_NAME);

			assertThat(exists).isFalse();
		}
	}

	@DisplayName("Create index")
	@Nested
	class TestCreateIndex {

		@AfterEach
		void cleanup() {
			deleteIndex();
		}

		@SneakyThrows
		@Test
		void shouldCreateIndex() {
			service.createIndex(INDEX_NAME);

			assertThat(existsIndex()).isTrue();
		}
	}

	@DisplayName("Exists security role")
	@Nested
	class TestExistsSecurityRole {

		@DisplayName("on existing")
		@Nested
		class TestOnExisting {

			@SneakyThrows
			@BeforeEach
			public void initSecurityRole() {
				createIndex();
				client.security().putRole(service.createPutRoleRequest(PutRoleRequestDataTestFactory.create()));
			}

			@AfterEach
			void cleanup() {
				deleteIndex();
			}
		}
	}

	@DisplayName("Create security role")
	@Nested
	class TestCreateSecurityRole {

		@AfterEach
		void cleanup() {
			deleteSecurityRole();
		}

		@SneakyThrows
		@Test
		void shouldCreateSecurityRole() {
			service.createOrUpdateSecurityRole(PutRoleRequestDataTestFactory.create());

			assertThat(existsSecurityRole()).isTrue();
		}

		@SneakyThrows
		private void deleteSecurityRole() {
			client.security().deleteRole(builder -> builder.name(PutRoleRequestDataTestFactory.NAME));
		}
	}

	@DisplayName("Exists security user")
	@Nested
	class TestExistsSecurityUser {

		@DisplayName("on existing")
		@Nested
		class TestOnExisting {

			@SneakyThrows
			@BeforeEach
			private void initSecurityUser() {
				createIndex();
				client.security().putUser(service.createPutUserRequest(PutUserRequestDataTestFactory.create()));
			}

			@AfterEach
			void cleanup() {
				deleteIndex();
				deleteSecurityRole();
			}
		}
	}

	@SneakyThrows
	private void deleteSecurityRole() {
		client.security().deleteUser(builder -> builder.username(PutUserRequestDataTestFactory.USERNAME));
	}

	@SneakyThrows
	private void deleteIndex() {
		client.indices().delete(builder -> builder.index(INDEX_NAME));
	}

	@DisplayName("Create security user")
	@Nested
	class TestCreateSecurityUser {

		@AfterEach
		void cleanup() {
			deleteSecurityUser();
		}

		@SneakyThrows
		@Test
		void shouldCreateSecurityUser() {
			service.createOrUpdateSecurityUser(PutUserRequestDataTestFactory.create());

			assertThat(existsSecurityUser()).isTrue();
		}

		@SneakyThrows
		private boolean existsSecurityUser() {
			return !client.security().getUser(builder -> builder.username(PutUserRequestDataTestFactory.USERNAME)).result().isEmpty();
		}

		@SneakyThrows
		private void deleteSecurityUser() {
			client.security().deleteUser(builder -> builder.username(PutUserRequestDataTestFactory.USERNAME));
		}
	}

	@DisplayName("Delete index")
	@Nested
	class TestDeleteIndex {

		@BeforeEach
		void init() {
			createIndex();
		}

		@SneakyThrows
		@Test
		void shouldDeleteIfExists() {
			service.deleteIndex(INDEX_NAME);

			assertThat(existsIndex()).isFalse();
		}
	}

	@DisplayName("Delete security role")
	@Nested
	class TestDeleteSecurityRole {

		@BeforeEach
		void init() {
			createIndex();
			createSecurityRole();
		}

		@AfterEach
		void cleanup() {
			deleteIndex();
		}

		@SneakyThrows
		@Test
		void shouldDeleteIfExists() {
			assertThat(existsSecurityRole()).isTrue();

			service.deleteSecurityRole(PutRoleRequestDataTestFactory.NAME);

			assertThat(existsSecurityRole()).isFalse();
		}
	}

	@DisplayName("Delete security user")
	@Nested
	class TestDeleteSecurityUser {

		@BeforeEach
		void init() {
			createIndex();
			createSecurityUser();
		}

		@AfterEach
		void cleanup() {
			deleteIndex();
		}

		@SneakyThrows
		@Test
		void shouldDeleteIfExists() {
			assertThat(existsSecurityUser()).isTrue();

			service.deleteSecurityUser(PutUserRequestDataTestFactory.USERNAME);

			assertThat(existsSecurityUser()).isFalse();
		}
	}

	@SneakyThrows
	private boolean existsIndex() {
		return client.indices().exists(ExistsRequest.of(builder -> builder.index(INDEX_NAME))).value();
	}

	@SneakyThrows
	private void createIndex() {
		client.indices().create(builder -> builder.index(INDEX_NAME));
	}

	@SneakyThrows
	private void createSecurityRole() {
		client.security().putRole(this::buildRequest);
	}

	private PutRoleRequest.Builder buildRequest(PutRoleRequest.Builder requestBuilder) {
		requestBuilder.name(PutRoleRequestDataTestFactory.NAME);
		requestBuilder.indices(this::buildIndicesPrivilegesRequest);
		return requestBuilder;
	}

	private IndicesPrivileges.Builder buildIndicesPrivilegesRequest(IndicesPrivileges.Builder builder) {
		builder.names(IndicesPrivilegesDataTestFactory.NAME);
		builder.privileges(IndicesPrivilegesDataTestFactory.PRIVILEGES);

		return builder;
	}

	@SneakyThrows
	private boolean existsSecurityRole() {
		return !client.security().getRole(builder -> builder.name(PutRoleRequestDataTestFactory.NAME)).result().isEmpty();
	}

	@SneakyThrows
	private void createSecurityUser() {
		client.security().putUser(this::buildPutUserRequest);
	}

	private PutUserRequest.Builder buildPutUserRequest(PutUserRequest.Builder builder) {
		builder.username(PutUserRequestDataTestFactory.USERNAME);
		builder.roles(PutUserRequestDataTestFactory.ROLES);
		builder.password(PutUserRequestDataTestFactory.PASSWORD);

		return builder;
	}

	@SneakyThrows
	private boolean existsSecurityUser() {
		return !client.security().getUser(builder -> builder.username(PutUserRequestDataTestFactory.USERNAME)).result().isEmpty();
	}
}